/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: csconn.c
 *
 * Purpose: This program connects to target process and gets a TCP socket Fd
 *          based on the passed parameters: host name and TCP port number.
 *          It updates the current association Ids socket_no and socket_status.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Jim Saunders & Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 * Notes:
 *
 * $Id: csconn.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: csconn.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <memory.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "ir.h"
#include "irconst.h"
#include "irassoc.h"
#include "irerror.h"

extern int errno;

/*****************************************************************************
 *
 * Function: cs_connect
 *
 * Purpose: This function connects to target process and gets a TCP port Fd
 *          based on the passed parameters: host name and TCP port number and
 *          updates the current association Ids socket_no[QRMSOCK]=socketFd
 *          and socket_status[QRMSOCK]=CS_CONNECTED.
 *
 * Input Arguments:     TCPPort    - TCP port number
 *                      hostname   - host name
 *
 * Return values:   SUCCESS - operation was completed successfully
 *                  FAILED  - operation failed
 *
 * Output Arguments: assocId - pointer to the current association
 *
 * External Variables:
 *
 * Function used:       socket()
 *                      gethostbyname()
 *                      connect()
 *                      inet_network()
 *                      inet_ntoa()
 *                      htons()
 *                      display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Jim Saunders & Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 ****************************************************************************/

int
cs_connect(TCPPort, hostname, assocId)
    int TCPPort;
    char *hostname;
    struct assoc_struct **assocId;
{
    struct sockaddr_in serverAddress;
    struct hostent *hostInfo;
    int sockfd;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling cs_connect ...\n");
#endif

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_connect: TCPPort=%d, hostname=%s\n", TCPPort, hostname);
#endif

    bzero((char *) &serverAddress, sizeof(serverAddress));
    serverAddress.sin_family = AF_INET;

    hostInfo = gethostbyname(hostname);

    if (hostInfo != NULL)
    {

#ifdef VMS
        serverAddress.sin_addr.s_addr = INADDR_ANY;
#else
        serverAddress.sin_addr.s_addr = inet_network(inet_ntoa(*hostInfo->h_addr_list));

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_connect: s_addr = %lu\n", serverAddress.sin_addr.s_addr);

#endif


#endif
    }
    else
    {
        display_error("KERNEL", ERR_CANNOT_GET_HOST_NAME, errno);
        return (FAILED);
    }

    serverAddress.sin_port = htons(TCPPort);

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "cs_connect: sin_port = %d, TCPort = %d\n", serverAddress.sin_port, TCPPort);
#endif

    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
        display_error("KERNEL", ERR_CREATE_SOCKET, errno);
        return (FAILED);
    }

    if (connect(sockfd, (struct sockaddr *) & serverAddress, sizeof(serverAddress)) < 0)
    {
        display_error("KERNEL", ERR_ACCEPT_CONNECT, errno);
        return (FAILED);
    }
#ifdef DEBUG_LEVEL1
    else
        fprintf(DEBUG_FILE1, "cs_connect: connected successfully\n");
#endif

    /* update the socket number */
    (*assocId)->socket_no[QRMSOCK] = sockfd;
    (*assocId)->socket_status[QRMSOCK] = CS_CONNECTED;

    return (SUCCESS);

}                                /* End Of Procedure cs_connect */
