/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpsearch.c
 *
 * Purpose: This is the target process black box search program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpsearch.c,v 1.4 1993/11/04 19:33:52 irsuprt Exp irsuprt $
 * $Log: tpsearch.c,v $
 * Revision 1.4  1993/11/04  19:33:52  irsuprt
 * added #ifdef DEAN_WILDER_FORMAT to skip 24 bytes between leader and Tag info
 * of Marc Record. (PROBLEM 19.0)
 *
 * Revision 1.3  1993/11/03  18:23:09  irsuprt
 * allowed searchReq->smallSetUpperBound > searchReq->mediumSetPresentNum (P 19)
 *
 * Revision 1.2  1993/11/03  16:02:03  irsuprt
 * free SEARCH_REFERENCE_ID (PROBLEM 20)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "ir_oid.h"
#include "asn_incl.h"
#include "irerror.h"
#include "tpconst.h"
#include "tpbbox.h"
#include "oid_tool.c"

AsnInt SEARCH_REFERENCE_ID_LENGTH;
char *SEARCH_REFERENCE_ID;

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_search_response
 *
 * Purpose: This function fills and creates the search response PDU. The
 *          DB record is filled with the data file 'canmarc'.
 *
 * Input Arguments:
 *          search_result -   0 = No error (DB record)
 *                            8 = Mandatory fields are invalid (Diag record)
 *                           31 = Resource control failed (Diag record)
 *                          104 = Access control failed (Diag record)
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_ptr - search response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_search_response(search_result, PDU_ptr)
    int search_result;
    APDU *PDU_ptr;
{

    NamePlusRecord *data_ptr;
    unsigned char temp_data[2000];
    int i, ch, max_rec_size;
    FILE *infile;
    char error_str[73];

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_search_response\n");
#endif

    PDU_ptr->a.searchResp = (struct SearchResp *) malloc(sizeof(struct SearchResp));

    PDU_ptr->choiceId = APDU_SEARCHRESP;

    PDU_ptr->a.searchResp->referenceId.octetLen = SEARCH_REFERENCE_ID_LENGTH;
    PDU_ptr->a.searchResp->referenceId.octs = (char *) malloc(SEARCH_REFERENCE_ID_LENGTH);
    memcpy(PDU_ptr->a.searchResp->referenceId.octs, SEARCH_REFERENCE_ID, SEARCH_REFERENCE_ID_LENGTH);

    PDU_ptr->a.searchResp->nextResultSetPosition = 0L;
    PDU_ptr->a.searchResp->searchStatus = 1L;

    PDU_ptr->a.searchResp->presentStatus = (AsnInt *) malloc(sizeof(AsnInt));
    *PDU_ptr->a.searchResp->presentStatus = 0;

    /* creating the query result record */

    PDU_ptr->a.searchResp->dbOrDiagnosticRec = (struct Records *) malloc(sizeof(struct Records));
    memset(PDU_ptr->a.searchResp->dbOrDiagnosticRec, NULL, sizeof(struct Records));

    /* if it's a DB record, create a DB record; else, create error record */
    if (search_result == 0)
    {
        PDU_ptr->a.searchResp->resultCount = 2L;
        PDU_ptr->a.searchResp->numRecordsReturned = 2L;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics = (AsnList *) malloc(sizeof(AsnList));

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->choiceId = RECORDS_DBORSURDIAGNOSTICS;

        /********** creating first record **********/
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first = (AsnListNode *) malloc(sizeof(AsnListNode));

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last = PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->curr = PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->count = 1;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first->prev = NULL;
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first->next = NULL;

        data_ptr = (NamePlusRecord *) malloc(sizeof(NamePlusRecord));
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first->data = data_ptr;

        data_ptr->databaseName.octetLen = 4;
        data_ptr->databaseName.octs = (char *) malloc(4);
        memcpy(data_ptr->databaseName.octs, "auls", 4);

        data_ptr->namePlusRecordChoice = (NamePlusRecordChoice *) malloc(sizeof(NamePlusRecordChoice));
        data_ptr->namePlusRecordChoice->choiceId = NAMEPLUSRECORDCHOICE_DATABASERECORD;
        data_ptr->namePlusRecordChoice->a.databaseRecord = (EXTERNAL *) malloc(sizeof(EXTERNAL));

        ASN_MakeOid((char *) OID_REC_SYN_USMARC, &data_ptr->namePlusRecordChoice->a.databaseRecord->direct_reference);

        data_ptr->namePlusRecordChoice->a.databaseRecord->indirect_reference = (AsnInt *) malloc(sizeof(AsnInt));
        *data_ptr->namePlusRecordChoice->a.databaseRecord->indirect_reference = 0;

        data_ptr->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octetLen = 0;
        data_ptr->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octs = NULL;

        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding = (struct EXTERNALChoice *) malloc(sizeof(struct EXTERNALChoice));
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned = (AsnOcts *) malloc(sizeof(AsnOcts));

        if ((infile = fopen("canmarc", "rb")) == NULL)
        {
            printf("BBOX: tp_fill_search_response: canmarc not found\n");
            return (FAILED);
        }

        /* skip 14 bytes */
        for (i = 0; i < 14; i++)
            ch = fgetc(infile);

        /* skip next 12 bytes */
        for (i = 0; i < 12; i++)
            ch = fgetc(infile);

        /* skip next 24 bytes */
        for (i = 0; i < 24; i++)
            ch = fgetc(infile);

#ifdef DEAN_WILDER_FORMAT
        /* skip next 24 bytes */
        for (i = 0; i < 24; i++)
            ch = fgetc(infile);
#endif

        i = 0;
        while ((ch = fgetc(infile)) != EOF)
            if (ch == 0x1d)
            {
                temp_data[i++] = ch;
                break;
            }
            else
                temp_data[i++] = ch;

        max_rec_size = i;

        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen = i;
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs = (char *) malloc(i);
        memcpy(data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs, temp_data, data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen);

        /********** creating second record **********/
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last = (AsnListNode *) malloc(sizeof(AsnListNode));

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->count = 2;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last->prev = PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->curr;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last->next = NULL;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->first->next = PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last;
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->curr = PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last;

        data_ptr = (NamePlusRecord *) malloc(sizeof(NamePlusRecord));
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->last->data = data_ptr;

        data_ptr->databaseName.octetLen = 4;
        data_ptr->databaseName.octs = (char *) malloc(4);
        memcpy(data_ptr->databaseName.octs, "auls", 4);

        data_ptr->namePlusRecordChoice = (NamePlusRecordChoice *) malloc(sizeof(NamePlusRecordChoice));
        data_ptr->namePlusRecordChoice->choiceId = NAMEPLUSRECORDCHOICE_DATABASERECORD;
        data_ptr->namePlusRecordChoice->a.databaseRecord = (EXTERNAL *) malloc(sizeof(EXTERNAL));

        ASN_MakeOid((char *) OID_REC_SYN_USMARC, &data_ptr->namePlusRecordChoice->a.databaseRecord->direct_reference);

        data_ptr->namePlusRecordChoice->a.databaseRecord->indirect_reference = (AsnInt *) malloc(sizeof(AsnInt));
        *data_ptr->namePlusRecordChoice->a.databaseRecord->indirect_reference = 0;

        data_ptr->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octetLen = 0;
        data_ptr->namePlusRecordChoice->a.databaseRecord->data_value_descriptor.octs = NULL;

        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding = (struct EXTERNALChoice *) malloc(sizeof(struct EXTERNALChoice));
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->choiceId = EXTERNALCHOICE_OCTET_ALIGNED;
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned = (AsnOcts *) malloc(sizeof(AsnOcts));

        /* skip next 36 bytes */
        for (i = 0; i < 36; i++)
            ch = fgetc(infile);

#ifdef DEAN_WILDER_FORMAT
        /* skip next 24 bytes */
        for (i = 0; i < 24; i++)
            ch = fgetc(infile);
#endif

        i = 0;
        while ((ch = fgetc(infile)) != EOF)
            if (ch == 0x1d)
            {
                temp_data[i++] = ch;
                break;
            }
            else
                temp_data[i++] = ch;

        if (i > max_rec_size)
            max_rec_size = i;

        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen = i;
        data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs = (char *) malloc(i);
        memcpy(data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octs, temp_data, data_ptr->namePlusRecordChoice->a.databaseRecord->encoding->a.octet_aligned->octetLen);

        fclose(infile);

        /* update the largest data record size */
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.dbOrSurDiagnostics->dataSize = max_rec_size;
    }
    else
    {
        PDU_ptr->a.searchResp->resultCount = 1L;
        PDU_ptr->a.searchResp->numRecordsReturned = 1L;

        /* create diagnostics record (error record) */
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics = (struct DiagRec *) malloc(sizeof(struct DiagRec));

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->choiceId = RECORDS_NONSURDIAGNOSTICS;

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen = strlen(OID_DIAGNOSTIC_SET_BIB1);
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs = (char *) malloc(PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);
        memcpy(PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octs, OID_DIAGNOSTIC_SET_BIB1, PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->diagnosticSetId.octetLen);

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->condition = search_result;        /* error code */

        switch (search_result)
        {
            case 8:
                strcpy(error_str, "THE MANDATORY FIELDS ARE INVALID");
                break;
            case 31:
                strcpy(error_str, "RESOURCE CONTROL FAILED");
                break;
            case 104:
                strcpy(error_str, "ACCESS CONTROL FAILED");
                break;
        }

        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen = strlen(error_str);
        PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs = (char *) malloc(PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);
        memcpy(PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octs, error_str, PDU_ptr->a.searchResp->dbOrDiagnosticRec->a.nonSurDiagnostics->addInfo.octetLen);
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_create_search_response
 *
 * Purpose: This function creates the search response PDU, encodes, and
 *          creates the search response APDU for sending to IR kernel.
 *
 * Input Arguments:
 *          search_result -   0 = No error (DB record)
 *                            8 = Mandatory fields are invalid (Diag record)
 *                           31 = Resource control failed (Diag record)
 *                          104 = Access control failed (Diag record)
 *          data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_search_response()
 *              InitNibbleMem()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *              FreeSearchResp()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_search_response(search_result, outputBuf, data_ptr, APDU_length)
    int search_result;
    SBuf *outputBuf;
    char **data_ptr;
    int *APDU_length;
{
    APDU PDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_search_response()\n");
#endif

    tp_fill_search_response(search_result, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX: tp_create_search_response, begin dump data:\n");
    PrintSearchResp(DEBUG_FILE2, PDU_ptr.a.searchResp, 0);
    fprintf(DEBUG_FILE2, "BBOX: tp_create_search_response, end dump data:\n");
#endif

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:INIT APDU Dump from tp_bbox\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    FreeSearchResp(PDU_ptr.a.searchResp);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_check_search_request
 *
 * Purpose: This function checks the search request PDU to make sure the
 *          mandatory fields are search and valid.
 *
 * Input Arguments: PDU_ptr - search request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_check_search_request(PDU_ptr)
    APDU *PDU_ptr;
{

    /* save reference ID */
    SEARCH_REFERENCE_ID_LENGTH = PDU_ptr->a.searchReq->referenceId.octetLen;
    free(SEARCH_REFERENCE_ID);
    SEARCH_REFERENCE_ID = (char *) malloc(PDU_ptr->a.searchReq->referenceId.octetLen);
    memcpy(SEARCH_REFERENCE_ID, PDU_ptr->a.searchReq->referenceId.octs, PDU_ptr->a.searchReq->referenceId.octetLen);

    if (PDU_ptr->a.searchReq->smallSetUpperBound == 0 &&
            PDU_ptr->a.searchReq->largeSetLowerBound == 1)
    {
        /*
         * The client system doesn't want to include the result in the
         * response
         */
    }
    else
    {
        if (PDU_ptr->a.searchReq->smallSetUpperBound <= 0)
        {
            display_error("BBOX", ERR_SEARCH_SMALL_SET_IS_INVALID, -1);
            return (FAILED);
        }

        if (PDU_ptr->a.searchReq->largeSetLowerBound <= 0)
        {
            display_error("BBOX", ERR_SEARCH_LARGE_SET_IS_INVALID, -1);
            return (FAILED);
        }

        if ((PDU_ptr->a.searchReq->smallSetUpperBound >= PDU_ptr->a.searchReq->largeSetLowerBound) ||
                (PDU_ptr->a.searchReq->smallSetUpperBound > PDU_ptr->a.searchReq->mediumSetPresentNum))
        {
            display_error("BBOX", ERR_SEARCH_SMALL_SET_IS_TOO_LARGE, -1);
            return (FAILED);
        }

        if (PDU_ptr->a.searchReq->mediumSetPresentNum >= PDU_ptr->a.searchReq->largeSetLowerBound)
        {
            display_error("BBOX", ERR_SEARCH_MEDIUM_SET_IS_TOO_LARGE, -1);
            return (FAILED);
        }
    }

    if (!(PDU_ptr->a.searchReq->replaceIndicator == 0 || PDU_ptr->a.searchReq->replaceIndicator == 1))
    {
        display_error("BBOX", ERR_INVALID_SEARCH_REPLACE_INDICATOR, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.searchReq->resultSetName.octetLen <= 0 || PDU_ptr->a.searchReq->resultSetName.octs == NULL)
    {
        display_error("BBOX", ERR_EMPTY_SEARCH_RESULT_SET_NAME, -1);
        return (FAILED);
    }

    if ((PDU_ptr->a.searchReq->databaseNames->count <= 0) ||
            (PDU_ptr->a.searchReq->databaseNames->dataSize <= 0) ||
            (PDU_ptr->a.searchReq->databaseNames->first == NULL)
        )
    {
        display_error("BBOX", ERR_EMPTY_SEARCH_DATABASE_NAME, -1);
        return (FAILED);
    }
    else if (PDU_ptr->a.searchReq->databaseNames->first->data == NULL)
    {
        display_error("BBOX", ERR_EMPTY_SEARCH_DATABASE_NAME, -1);
        return (FAILED);
    }

    if (!(PDU_ptr->a.searchReq->query->choiceId == QUERY_TYPE_0 ||
          PDU_ptr->a.searchReq->query->choiceId == QUERY_TYPE_1 ||
          PDU_ptr->a.searchReq->query->choiceId == QUERY_TYPE_2 ||
          PDU_ptr->a.searchReq->query->choiceId == QUERY_TYPE_100 ||
          PDU_ptr->a.searchReq->query->choiceId == QUERY_TYPE_101
          )
        )
    {
        display_error("BBOX", ERR_INVALID_SEARCH_QUERY_TYPE, -1);
        return (FAILED);
    }

    switch (PDU_ptr->a.searchReq->query->choiceId)
    {
        case QUERY_TYPE_0:
            if (PDU_ptr->a.searchReq->query->a.type_0->octetLen <= 0 ||
                    PDU_ptr->a.searchReq->query->a.type_0->octs == NULL)
            {
                display_error("BBOX", ERR_EMPTY_SEARCH_QUERY, -1);
                return (FAILED);
            }
            break;
        case QUERY_TYPE_1:
            if (PDU_ptr->a.searchReq->query->a.type_1->attributeSetId.octetLen <= 0 ||
                    PDU_ptr->a.searchReq->query->a.type_1->attributeSetId.octs == NULL ||
                PDU_ptr->a.searchReq->query->a.type_1->rPNStructure == NULL)
            {
                display_error("BBOX", ERR_EMPTY_SEARCH_QUERY, -1);
                return (FAILED);
            }
            break;
        case QUERY_TYPE_2:
            if (PDU_ptr->a.searchReq->query->a.type_2->octetLen <= 0 ||
                    PDU_ptr->a.searchReq->query->a.type_2->octs == NULL)
            {
                display_error("BBOX", ERR_EMPTY_SEARCH_QUERY, -1);
                return (FAILED);
            }
            break;
        case QUERY_TYPE_100:
            if (PDU_ptr->a.searchReq->query->a.type_100->octetLen <= 0 ||
                    PDU_ptr->a.searchReq->query->a.type_100->octs == NULL)
            {
                display_error("BBOX", ERR_EMPTY_SEARCH_QUERY, -1);
                return (FAILED);
            }
            break;
        case QUERY_TYPE_101:
            if (PDU_ptr->a.searchReq->query->a.type_101->octetLen <= 0 ||
                    PDU_ptr->a.searchReq->query->a.type_101->octs == NULL)
            {
                display_error("BBOX", ERR_EMPTY_SEARCH_QUERY, -1);
                return (FAILED);
            }
            break;
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_search_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the search request PDU,
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the search response APDU to IR kernel.
 *
 * Input Arguments:
 *              PDU - search request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_check_search_request()
 *              FreeSearchReq()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_search_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_search_request(PDU, data_ptr, socketFd)
    APDU *PDU;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_search_request\n");
#endif

    result = tp_check_search_request(PDU);
    FreeSearchReq(PDU->a.searchReq);

    if (result == SUCCESS)
    {

        if (tp_access_control(socketFd, data_ptr, SEARCH_REFERENCE_ID_LENGTH, SEARCH_REFERENCE_ID) == SUCCESS)
        {
            if (tp_resource_control(socketFd, data_ptr, SEARCH_REFERENCE_ID_LENGTH, SEARCH_REFERENCE_ID) == FAILED)
            {
                display_error("BBOX", ERR_RESOURCE_CONTROL_FAILED, -1);
                tp_create_search_response(31, &APDU_ptr, data_ptr, &APDU_length);
                return (FAILED);
            }
            else
                tp_create_search_response(0, &APDU_ptr, data_ptr, &APDU_length);        /* success */
        }
        else
        {
            display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
            tp_create_search_response(104, &APDU_ptr, data_ptr, &APDU_length);
        }
    }
    else
    {
        /* create the diagnostic record due to incomplete mandatory fields */
        tp_create_search_response(8, &APDU_ptr, data_ptr, &APDU_length);
    }

    if (tpa_send_APDU(socketFd, &APDU_ptr, (int) APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

#ifdef DEBUG_LEVEL1
    else
        fprintf(DEBUG_FILE1, "BBOX: tp_process_search_request: %ld search response sent\n", APDU_length);
#endif

    return (SUCCESS);
}
